/** @odoo-module **/

import { registerModel } from '@mail/model/model_core';
import { one } from '@mail/model/model_field';
import { clear } from '@mail/model/model_field_command';

registerModel({
    name: 'ThreadIconView',
    identifyingMode: 'xor',
    fields: {
        chatWindowHeaderViewOwner: one('ChatWindowHeaderView', {
            identifying: true,
            inverse: 'threadIconView',
        }),
        discussSidebarCategoryItemOwner: one('DiscussSidebarCategoryItem', {
            identifying: true,
            inverse: 'threadIconView',
        }),
        discussSidebarMailboxViewOwner: one('DiscussSidebarMailboxView', {
            identifying: true,
            inverse: 'threadIconView',
        }),
        thread: one('Thread', {
            compute() {
                if (this.chatWindowHeaderViewOwner) {
                    return this.chatWindowHeaderViewOwner.chatWindowOwner.thread;
                }
                if (this.discussSidebarCategoryItemOwner) {
                    return this.discussSidebarCategoryItemOwner.thread;
                }
                if (this.discussSidebarMailboxViewOwner) {
                    return this.discussSidebarMailboxViewOwner.mailbox.thread;
                }
                if (this.threadViewTopbarOwner) {
                    return this.threadViewTopbarOwner.thread
                }
            },
            required: true,
        }),
        threadTypingIconView: one('ThreadTypingIconView', {
            compute() {
                if (
                    this.thread.channel &&
                    this.thread.channel.channel_type === 'chat' &&
                    this.thread.channel.correspondent &&
                    this.thread.orderedOtherTypingMembers.length > 0
                ) {
                    return {};
                }
                return clear();
            },
            inverse: 'threadIconViewOwner'
        }),
        threadViewTopbarOwner: one('ThreadViewTopbar', {
            identifying: true,
            inverse: 'threadIconView',
        }),
    },
});
